#!/usr/bin/perl -w

package Node;

sub new
{
	my ($class,%args)=@_;
	my $self = { 
			name=> $args{"name"} ,
			duration=> $args{"duration"} ,
			deps=>$args{"deps"} ,
		};
	bless $self,$class;
	return $self;
}

package main;
use Data::Dumper;

use Getopt::Std;
getopts("d",\%opts) or usage();
$Global::debug=$opts{'d'} || 0;

chomp (my @input=<ARGV>);
my (%nodes,@paths,@path);
parse(\@input,\%nodes) or die "Cannot parse input";
print Dumper \%nodes if  $Global::debug;
push @path,"FINAL";
push @paths,\@path;
generate_paths(\%nodes,\@paths,\@path,{});
print Dumper \@paths if  $Global::debug;
my %durations = map { $_ => duration_path(\%nodes,$_) } @paths;
print Dumper \%durations if  $Global::debug;
my @sorted_paths = map { $_->[0] } sort { $b->[1] <=> $a->[1] } map { [ $_ , $durations{$_} ] } @paths;
print Dumper \@sorted_paths if  $Global::debug;

if ($Global::debug) {
	foreach my $path (@paths) {
		print join "*",@{$path},duration_path(\%nodes,$path),"\n";
	}
}

my @critical_paths = grep { $durations{$_} == $durations{$sorted_paths[0]} } @sorted_paths; 
#print "Critical Path".((@critical_paths > 1) ? "s":"")." (Duration = ".$durations{$sorted_paths[0]}."):\n";
print map { join "",map { "$_\n" } reverse(@{$_}), '' } @critical_paths;

sub parse
{
	my ($input_r,$nodes_r)=@_;
	foreach (@$input_r) {
		my ($name,$duration,@deps)=split /\s+/,$_;
		next if not defined($name);
		my $node = Node->new(name=>$name,duration=>$duration,deps=>\@deps);
		$nodes_r->{$name}=$node;
	}
	return 1;
}

sub generate_paths
{
	my ($nodes_r, $paths_r, $path_r, $painted_node_r)=@_;
	my $last_node=$nodes_r->{$path_r->[-1]};
	print "Last node:",$last_node->{"name"},"\n" if $Global::debug;
	print "Last node deps:",join "*",@{$last_node->{"deps"}},"\n" if $Global::debug;
	my @deps=@{$last_node->{"deps"}};
	if (@deps != 0) {
		if ($painted_node_r->{$last_node}++) {
			die "Circular Reference detected\n";
		}
		my @start_path=@{$path_r};
		push @{$path_r}, shift @deps;
		generate_paths($nodes_r,$paths_r,$path_r,$painted_node_r);
		foreach (@deps) {
			my @new_path=@start_path;
			push @new_path,$_;
			push @$paths_r,\@new_path;
			generate_paths($nodes_r,$paths_r,\@new_path,$painted_node_r);
		}
		$painted_node_r->{$last_node}--;
	} else {
		print "End Point\n" if $Global::debug;
		return;
	}
}

sub duration_path
{
	my ($nodes_r,$path)=@_;
	my $duration=0;
	foreach (@$path) {
		$duration+=$nodes_r->{$_}{"duration"};
	}
	return $duration;
}
