# Compute Crtitical Path from a list of tasks provided in a
#     TASKNAME DELAY DEPENDENDIES format like :
#       EATING   1
#       WORKING  8 EATING
#       SLEEPING 8 WORKING
#       FINAL    0 SLEEPING EATING WOKING

use strict;
use warnings;

my %tasks = ();
my @task;
my $name;

# Read tasks and store task time and dependencies
while (<>) {
  chomp;
  @task = split;
  if (@task) {
    $name = shift @task;
    $tasks{$name}{delay} = shift @task;
    $tasks{$name}{dep} = [@task];
  }
}

# A "FINAL" task should be defined
die "No Definition for FINAL task\n" unless exists $tasks{FINAL};

# Compute cumulative ETC of every task (counting dependencies)
foreach (keys %tasks) {
  computeCompletion($_);
}

# Compute/Print Critical Path
printCriticalPath('FINAL');
print "\n";

#####################################################################
sub computeCompletion {
  my ($t) = @_;

  # The cumulative ETC is the sum of the task own completion time 
  # plus the largest cumulative ETC of its dependencies..

  # If the ETC has not already been computed
  if (!exists $tasks{$t}{completion}) {

    $tasks{$t}{completion} = -1;
    my $max = 0;
    # Find the largest ETC amongst the dependencies
    foreach my $d (@{$tasks{$t}{dep}}) {
      # Flag an error if a dependency doesn't exist
      die "Missing Dependency [$d]\n" unless exists $tasks{$d};
      my $r = computeCompletion($d);
      $max = $r if $r > $max;
    }
    $tasks{$t}{completion} = $tasks{$t}{delay} + $max;
  } elsif ($tasks{$t}{completion} < 0) {
    # Flag an error if a task depends on itself
    die "Circular dependency for Task $t\n";
  }
  return $tasks{$t}{completion};
}

######################################################################
sub printCriticalPath {
  my ($t) = @_;
  my $worst = undef;
  my $max = -1;

  # Critical Path is computed backward from the FINAL task
  # selecting dependencies with the largest cumulative ETC 
  # at every fork in the graph

  foreach my $d (@{$tasks{$t}{dep}} ) {
    if ($tasks{$d}{completion} > $max ) {
      $max = $tasks{$d}{completion};
      $worst = $d;
    }
  }
  printCriticalPath($worst) if defined $worst;

#  print "Task $t\tReady in : $tasks{$t}{completion} days!\n";
  print "$t\n";
}
