#!perl
    
while (<>) {
  chomp;
  my ($task, $duration, $dep) = split /\s+/, $_, 3;
  $depends_on{$task} = $dep;
  for (split /\s+/, $dep) {
    $depended_on_by{$_} .= "$task ";
  }
  $duration{$task} = $duration;
  $tasks++;
}

# Compute earliest start and end times for each task
my(%start, %end);
while (keys(%start) < $tasks) {
  my $num_done = keys %start;
 TASK:
  while (($task, $dep) = each %depends_on) {
    my $latest_end = 0;
    next TASK if exists $end{$task};
    for my $dependency (split /\s+/, $dep) {
      next TASK unless exists $end{$dependency};
      $latest_end = $end{$dependency} if $latest_end < $end{$dependency};
    }
    # The last dependency for task $task ends no later than $latest_end
    $start{$task} = $latest_end;
    $end{$task}   = $latest_end + $duration{$task};
  }
  if ($num_done == keys %start) {
    # Didn't make any progress; there must be a circular dependency
    die "Circular dependency.\n";
  }
}

# A task T is on the critical path if:
#  1. it is FINAL, or
#  2. a. a critical path task depends on it, and
#     b. the cp task start time is equal to T's end time
#
# A task T is NOT on the critical path if it is not FINAL
# and if all cp tasks that depend on it start later than T's end time.

my %crit_path = ('FINAL', 1);
while (keys(%crit_path) < $tasks) {
 TASK:
  while (($task, $dep) = each %depended_on_by) {
    next TASK if exists $crit_path{$task}; # Did this one already
    for my $dependency (split /\s+/, $dep) {
      next TASK unless exists $crit_path{$dependency}; # Not sure yet
      if ($crit_path{$dependency} && $start{$dependency} == $end{$task}) {
        $crit_path{$task} = 1;
        next TASK;
      }
    }
    $crit_path{$task} = 0;
  }
}

#print "Critical path:\n";
for (grep $crit_path{$_}, sort keys %crit_path) {
  print "$_\n";
}
print "\n";
