#!perl

my $DEBUG = 0;

while (<>) {
  chomp;
  my ($task, $duration, $dep) = split /\s+/, $_, 3;
  $depends_on{$task} = $dep;
  for (split /\s+/, $dep) {
    $depended_on_by{$_} .= "$task ";
  }
  $duration{$task} = $duration;
  $tasks++;
}

my @tasks = tsort(%depends_on);

# Topological sort of a hash
sub tsort {
  my %depends_on = @_;
  my %ordered;
  my @order;
  while (keys(%ordered) < keys(%depends_on)) {
    my $n_ordered = keys %ordered;
  TASK:
    while (my ($task, $dep) = each %depends_on) {
      next TASK if $ordered{$task};
      for (split /\s+/, $dep) {
        next TASK unless $ordered{$_};
      }
      # All dependencies ordered
      push @order, $task;
      $ordered{$task} = 1;
    }
    if ($n_ordered == keys %ordered) {
      die "Circular dependency\n";
    }
  }
  return @order;
}

print "tsort: @tasks\n" if $DEBUG;

# Compute earliest start time and end time for each task
for my $task (@tasks) {
  my $earliest_start = 0;
  for my $dep (split /\s+/, $depends_on{$task}) {
    next unless exists $end{$dep};
    next if $end{$dep} <= $earliest_start;
    $earliest_start = $end{$dep};
  }
  $start{$task} = $earliest_start;
  $end{$task}   = $earliest_start + $duration{$task};
  print "$task $start{$task}-$end{$task}\n" if $DEBUG;
}

# Find the earliest time at which a task depending on T can start
# If that's later than the expected end time for T, T has some slack 
# A task is on the critical path if (a) it is FINAL, or (b1)
# it has no slack, and (b2) a critical path task depends on it

$critpath{FINAL}=1;
for my $task (reverse @tasks) {
  next if $task eq "FINAL";
  my $earliest_start = 1000000000;
  my $maybe_critpath = 0;
  for my $dep (split /\s+/, $depended_on_by{$task}) {
    $maybe_critpath = 1 if $critpath{$dep};
    next if $start{$dep} >= $earliest_start;
    $earliest_start = $start{$dep};
  }
  $critpath{$task} = $maybe_critpath && $earliest_start == $end{$task};
  print "$task slack $slack{$task} ($critpath{$task})\n" if $DEBUG;
}

#print "Critical path:\n";
for  (@tasks) {
  print "$_\n" if $critpath{$_};
}
print "\n";
