#!/usr/bin/perl
# $Id$
use strict;
use warnings;


###
package Tasks;


sub new { bless {}, shift }


sub duration {
  my ($self, $task, $val) = @_;
  $self->{$task}->{duration} = $val if defined $val;
  return $self->{$task}->{duration};
}


sub earliest {
  my ($self, $task, $val) = @_;
  $self->{$task}->{earliest} = $val if defined $val;
  return $self->{$task}->{earliest};
}


sub latest {
  my ($self, $task, $val) = @_;
  $self->{$task}->{latest} = $val if defined $val;
  return $self->{$task}->{latest};
}


sub prev_tasks {
  my ($self, $task, $val) = @_;
  $self->{$task}->{prev_tasks} = $val if ref $val eq 'ARRAY';
  return @{$self->{$task}->{prev_tasks}};
}


sub total_duration { $_[0]->earliest('FINAL') }


sub read_tasks {
  my ($self, $fh) = @_;

  while (<$fh>) {
    (my ($task, $duration, @prev_tasks)) = split;
    $self->duration($task, $duration);
    $self->prev_tasks($task, \@prev_tasks);
  }

  die "Circular task chain found\n" if $self->find_circular;

}


sub find_circular {
  my ($self, $task, $chain) = @_;
  $task  ||= 'FINAL';
  $chain ||= [];
  
  for (@$chain) {
    return $task if $task eq $_;
  }

  push @$chain, $task;
  my $circ_ref_found;
  for ($self->prev_tasks($task)) {
    $circ_ref_found = $self->find_circular($_, $chain);
    last if $circ_ref_found;
  }
  pop @$chain;
  return $circ_ref_found;

}


sub calc_earliest_completion {
  my ($self, $task) = @_;
  $task ||= 'FINAL';

  return if defined $self->earliest($task);

  $self->calc_earliest_completion($_) for $self->prev_tasks($task);

  my $earliest;
  for ($self->prev_tasks($task)) {
    if ( ! defined $earliest || $earliest < $self->earliest($_) ) {
      $earliest = $self->earliest($_);
    }
  }
  $earliest ||= 0;
  $self->earliest($task, $earliest + $self->duration($task));

}


sub calc_latest_completion {
  my ($self, $task) = @_;
  $task ||= 'FINAL';

  $self->latest('FINAL', $self->earliest('FINAL'))
    if $task eq 'FINAL';
  
  my $latest = $self->latest($task) - $self->duration($task);
  for ($self->prev_tasks($task)) {

    if ( ! defined $self->latest($_) 
    || $latest < $self->latest($_) ) {

      $self->latest($_, $latest);

    }

  }

  $self->calc_latest_completion($_) for $self->prev_tasks($task);

}


sub find_crit_path {
  my ($self, $task) = @_;
  $task ||= 'FINAL';

  my @path;

  for ( $self->prev_tasks($task) ) {
    if ( $self->earliest($_) == $self->latest($_) ) {
      @path = $self->find_crit_path($_);
      last;
    }
  }

  return @path, $task;

}


###
package main;


my $pathfind = Tasks->new;
$pathfind->read_tasks(*ARGV);
$pathfind->calc_earliest_completion;
$pathfind->calc_latest_completion;

print map {"$_\n"} $pathfind->find_crit_path;
print "\n";
#printf "%s Total Duration\n", $pathfind->total_duration;


__END__
G 24
A 14
I 14
J 9
H 21 G
D 10 A
B 5 I
C 5 J
E 31 B C D
F 7 E
K 21 F
FINAL 0 H K
