#!/usr/bin/perl

use strict;
use warnings;

use List::Util qw(max);

# process input
my %tasks = ();
while (<>) {
     chomp;
     my ($label, $estimation, @dependencies) = split;
     $tasks{$label}{estimation} = $estimation;
     $tasks{$label}{dependencies} = \@dependencies;
}

# compute and print critical paths
my @cpaths = critical_paths('FINAL');
#print "@{$_->{tasks}}\n" for @cpaths;
for my $cpath (@cpaths) {
  print "$_\n" for @{$cpath->{tasks}}, '';
}


# stores the path being traversed, shared by all recursive calls
my @stack = ();
sub critical_paths {
     my $task = shift;

     if (grep { $task eq $_ } @stack) {
         die "circular dependency: $task @{[reverse @stack]}\n";
     }

     my @paths = ();
     if (@{$tasks{$task}{dependencies}}) {
         push @stack, $task;
         foreach my $dep (@{$tasks{$task}{dependencies}}) {
             push @paths, critical_paths($dep);
         }
         pop @stack;
     } else {
         push @paths, {tasks => [], estimation => 0};
     }

     # keep those paths with the highest estimation
     my $max = max map $_->{estimation}, @paths;
     my @cpaths = grep { $_->{estimation} == $max } @paths;
     foreach my $cpath (@cpaths) {
         push @{$cpath->{tasks}}, $task;
         $cpath->{estimation} += $tasks{$task}{estimation};
     }

     return @cpaths;
}


